/*
 *   Copyright (c) International Business Machines  Corp., 2000
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or 
 *   (at your option) any later version.
 * 
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software 
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef _H_JFS_LOCK
#define _H_JFS_LOCK

#include <linux/spinlock.h>
#include <linux/sched.h>

/*
 *	jfs_lock.h
 *
 * JFS lock definition for globally referenced locks
 */

/*
 * General purpose spinlock for really quick stuff
 */
extern spinlock_t jfs_spinlock;

/* readers/writer lock: thread-thread */
/*
 * see jfs_txnmgr.h.  Since jfs_lock.h is included first, we need to define
 * IDIRTY here
 */
#define IDIRTY	0x0080		/* Tried to write dirty inode while locked */

/*
 * RW semaphores do not currently have a trylock function.  Since the
 * implementation varies by platform, I have implemented a platform-independent
 * wrapper around the rw_semaphore routines.  If this turns out to be the best
 * way of avoiding our locking problems, I will push to get a trylock
 * implemented in the kernel.
 */
#define RDWRLOCK_T jfs_rwlock_t
#define RDWRLOCK_ALLOC(Lock,Flag,Class,Occurrence)
#define RDWRLOCK_FREE(Lock)
static inline void RDWRLOCK_INIT(jfs_rwlock_t * Lock)
{
	init_rwsem(&Lock->rw_sem);
	atomic_set(&Lock->in_use, 0);
}
static inline void READ_LOCK(jfs_rwlock_t * Lock)
{
	down_read(&Lock->rw_sem);
	atomic_inc(&Lock->in_use);
}
static inline void READ_UNLOCK(struct inode *ip, jfs_rwlock_t * Lock)
{
	atomic_dec(&Lock->in_use);
	up_read(&Lock->rw_sem);
	if (atomic_read(&Lock->in_use) && (JFS_IP(ip)->flag & IDIRTY)) {
		mark_inode_dirty(ip);
		JFS_IP(ip)->flag &= ~IDIRTY;
	}
}
static inline void WRITE_LOCK(jfs_rwlock_t * Lock)
{
	down_write(&Lock->rw_sem);
	atomic_inc(&Lock->in_use);
}

/*
 * I'm not worried about a race condition between processors.  The problem
 * I'm avoiding is trying to take a nested lock when memory pressure forces
 * the kernel to try to write all the dirty inodes.
 */
static inline int WRITE_TRYLOCK(jfs_rwlock_t * Lock)
{
	if (atomic_read(&Lock->in_use))
		return 0;
	WRITE_LOCK(Lock);
	return 1;
}
static inline void WRITE_UNLOCK(struct inode *ip, jfs_rwlock_t * Lock)
{
	atomic_dec(&Lock->in_use);
	up_write(&Lock->rw_sem);
	if (atomic_read(&Lock->in_use) && (JFS_IP(ip)->flag & IDIRTY)) {
		mark_inode_dirty(ip);
		JFS_IP(ip)->flag &= ~IDIRTY;
	}
}

#define IREAD_LOCK(ip)		READ_LOCK(&JFS_IP(ip)->rdwrlock)
#define IREAD_UNLOCK(ip)	READ_UNLOCK(ip, &JFS_IP(ip)->rdwrlock)
#define IWRITE_LOCK(ip)		WRITE_LOCK(&JFS_IP(ip)->rdwrlock)
#define IWRITE_TRYLOCK(ip)	WRITE_TRYLOCK(&JFS_IP(ip)->rdwrlock)
#define IWRITE_UNLOCK(ip)	WRITE_UNLOCK(ip, &JFS_IP(ip)->rdwrlock)
#define IWRITE_LOCK_LIST	iwritelocklist

void iwritelocklist(int, ...);

#endif				/* _H_JFS_LOCK */
